/*******************************************************************************
 * Copyright (c) 2006, 2014 Tom Schindl and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Tom Schindl <tom.schindl@bestsolution.at> - initial API and implementation
 *     Florian Potschka <signalrauschen@gmail.com> - Bug 260061
 *     Alexander Ljungberg <siker@norwinter.com> - Bug 260061
 *     Jeanderson Candido <http://jeandersonbc.github.io> - Bug 414565
 *     Alexander Mark Diewald <diewald@fortiss.org>
 *******************************************************************************/
package org.fortiss.tooling.base.ui.annotation.view;

import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.ui.annotation.AnnotationEntry;

/**
 * A {@link ColumnLabelProvider} for annotations that displays checkboxes indicating the value of
 * annotations that contain boolean values. This is a workaround using 'screenshots' of the
 * checkboxes, since {@link TableViewer} is not able to disply SWT widgets when a cell is not
 * edited. Since the screenshots are made during runtime, the resulting images fit the platform on
 * which AF3 is run.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class EmulatedNativeCheckBoxLabelProvider extends LabelProviderBase {
	/** Specification class of this column */
	protected Class<? extends IAnnotatedSpecification> specClass;

	/** String holding the key of which identifies the value to be displayed */
	protected String instanceKey;

	/** Key for the 'checked' checkbox. */
	private static final String CHECKED_KEY = "CHECKED";
	/** Key for the 'unchecked' checkbox. */
	private static final String UNCHECK_KEY = "UNCHECKED";

	/** Constructor. */
	public EmulatedNativeCheckBoxLabelProvider(Class<? extends IAnnotatedSpecification> specClass,
			IAnnotationViewPart viewPart, String instanceKey, ColumnViewer viewer) {
		super(viewPart);
		this.specClass = specClass;
		this.instanceKey = instanceKey;

		if(JFaceResources.getImageRegistry().getDescriptor(CHECKED_KEY) == null) {
			JFaceResources.getImageRegistry()
					.put(UNCHECK_KEY, makeShot(viewer.getControl(), false));
			JFaceResources.getImageRegistry().put(CHECKED_KEY, makeShot(viewer.getControl(), true));
		}
	}

	/** Takes a 'screenshot' of the checkbox control an stores it in the image registry. */
	private Image makeShot(Control control, boolean type) {
		// Hopefully no platform uses exactly this color because we'll make
		// it transparent in the image.
		Color greenScreen = new Color(control.getDisplay(), 222, 223, 224);

		Shell shell = new Shell(control.getShell(), SWT.NO_TRIM);

		// otherwise we have a default gray color
		shell.setBackground(greenScreen);

		Button button = new Button(shell, SWT.CHECK);
		button.setBackground(greenScreen);
		button.setSelection(type);

		// otherwise an image is located in a corner
		button.setLocation(1, 1);
		Point bsize = button.computeSize(SWT.DEFAULT, SWT.DEFAULT);

		// otherwise an image is stretched by width
		bsize.x = Math.max(bsize.x - 1, bsize.y - 1);
		bsize.y = Math.max(bsize.x - 1, bsize.y - 1);
		button.setSize(bsize);
		shell.setSize(bsize);

		shell.open();
		GC gc = new GC(shell);
		Image image = new Image(control.getDisplay(), bsize.x, bsize.y);
		gc.copyArea(image, 0, 0);
		gc.dispose();
		shell.close();

		ImageData imageData = image.getImageData();
		imageData.transparentPixel = imageData.palette.getPixel(greenScreen.getRGB());

		return new Image(control.getDisplay(), imageData);
	}

	/** {@inheritDoc} */
	@Override
	public String getText(Object element) {
		return "";
	}

	/** {@inheritDoc} */
	@Override
	public Image getImage(Object element) {
		if(element instanceof AnnotationEntry && ((AnnotationEntry)element).canEdit(specClass)) {
			if(isChecked(element)) {
				return JFaceResources.getImageRegistry().get(CHECKED_KEY);
			}
			return JFaceResources.getImageRegistry().get(UNCHECK_KEY);
		}
		return null;
	}

	/**
	 * Determines whether the boolean value is 'true' or 'false'. Used to detrmine the 'checked'
	 * status of the checkbox.
	 */
	protected boolean isChecked(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry annotationEntry = (AnnotationEntry)element;

			Object value = null;
			try {
				value = annotationEntry.getSpecificationValue(specClass, instanceKey);
			} catch(Exception e) {
				// Ignore. Return 'false' in this case.
			}
			if(value != null) {
				// It is already known that the value is a boolean.
				return (Boolean)value;
			}
		}
		return false;
	}
}
